from __future__ import unicode_literals

__version__ = '1.0.2'

from .dateutils import *  # NOQA
from .forms import *  # NOQA
from .qs import *  # NOQA
from .utils import *  # NOQA
from .views import *  # NOQA
from .widgets import *  # NOQA
from .xlsxutils import *  # NOQA



from __future__ import unicode_literals

import math
import importlib
from django.template.defaultfilters import floatformat
from django.contrib.humanize.templatetags.humanize import intcomma


__all__ = (
    'safe_eval', 'fmt_num', 'create_instance',
    'format_filesize',
)


def safe_eval(source, *args, **kwargs):
    """ eval without import """
    source = source.replace('import', '')  # import is not allowed
    return eval(source, *args, **kwargs)


def fmt_num(num, zero_num=None):
    """ humanize number(9000 to 9,000) """
    if zero_num is not None:
        num = floatformat(num, zero_num)
    return intcomma(num, False)


def create_instance(class_name, *args, **kwargs):
    """
    create class instance

    class_name: name of class i.e.: "django.http.HttpResponse"
    *args, **kwargs: param for class
    """
    idx = class_name.rindex(r'.')
    _module = importlib.import_module(class_name[:idx])
    class_name = class_name[idx + 1:]
    return getattr(_module, class_name)(*args, **kwargs)


def format_filesize(size):
    if (size < 1024):
        return '%s B' % size
    size = size / 1024.0
    size_name = ("KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB")
    i = int(math.floor(math.log(size, 1024)))
    p = math.pow(1024, i)
    s = round(size / p, 2)
    return '%s %s' % (s, size_name[i])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import json
from django.http import HttpResponse

try:
    from crispy_forms.helper import FormHelper
except ImportError:
    pass


__all__ = (
    'qdict_get_list', 'request_get_next', 'save_formset',
    'render_json', 'has_any_perm', 'get_post_data',
    'get_client_ip', 'create_formset', 'forms_is_valid',
)


def qdict_get_list(qdict, k):
    """
    get list from QueryDict and remove blank date from list.
    """
    pks = qdict.getlist(k)
    return [e for e in pks if e]


def request_get_next(request, default_next):
    """
    get next url form request

    order: POST.next GET.next HTTP_REFERER, default_next
    """
    next_url = request.POST.get('next')\
        or request.GET.get('next')\
        or request.META.get('HTTP_REFERER')\
        or default_next
    return next_url


def save_formset(formset, ext_vals):
    formset.save(commit=False)
    for f in formset.saved_forms:
        o = f.save(commit=False)
        for k, v in ext_vals.items():
            setattr(o, k, v)
        o.save()


def render_json(data, ensure_ascii=False, request=None, as_text=False):
    fmt = 'json'
    content_type = "application/json"
    if as_text:
        content_type = "text/html"
    plain = json.dumps(data, ensure_ascii=False)
    if request:
        fmt = request.GET.get('fmt', 'json')
        if fmt == "jsonp":
            jsonp_cb = request.GET.get('callback', 'callback')
            # content_type = "application/javascript"
            plain = "%s(%s);" % (jsonp_cb, plain)
    return HttpResponse(plain, content_type=content_type)


def has_any_perm(user, perms):
    if not isinstance(perms, [list, tuple]):
        perms = [e.strip() for e in perms.split(',') if e.strip()]
    for p in perms:
        if user.has_perm(p):
            return True
    return False


def get_post_data(request):
    """ get request.POST, if POST is empty return None """
    if request.method == 'POST':
        return request.POST
    return None


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip = x_forwarded_for.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip


def create_formset(
        formset_class, prefix,
        template_prefix=None, **kwargs):
    helper = FormHelper()
    project_name = ''
    template = 'bootstrap3/table_inline_formset.html'
    if template_prefix:
        template = "%s/%s" % (template_prefix, template)
    helper.template = template
    formset = formset_class(prefix=prefix, **kwargs)
    formset.helper = helper
    return formset


def forms_is_valid(forms):
    return all([form.is_valid() for form in forms])



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms

from lbutils import BootstrapFormHelperMixin
from lbutils import JustSelectedSelect
from lbutils import JustSelectedSelectMultiple
from .models import Book


class BookForm(BootstrapFormHelperMixin, forms.ModelForm):
    def __init__(self, *args, **kw):
        super(BookForm, self).__init__(*args, **kw)
        self.init_crispy_helper()

    def clean_name(self):
        name = self.cleaned_data.get('name')
        self.check_uniqe(Book, name=name)
        return name

    class Meta:
        model = Book
        fields = [
            'name', 'descn', 'price', 'is_active', 'category', 'authors'
        ]
        widgets = {
            'category': JustSelectedSelect,
            'authors': JustSelectedSelectMultiple
        }



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


from unittest import skipUnless
import django
from django.http import QueryDict
from django.test import TestCase

from lbutils import fmt_month
from lbutils import get_year_choices
from lbutils import fmt_num
from lbutils import create_instance
from lbutils import get_or_none
from lbutils import get_pk_or_none
from lbutils import get_sum
from lbutils import get_max
from lbutils import do_filter
from lbutils import QuickSearchForm
from lbutils import forms_is_valid
from lbutils import render_json
from lbutils import qdict_get_list
from lbutils import simple_export2xlsx
from lbutils import format_filesize
from lbutils.templatetags.lbutils import display_array
from lbutils.templatetags.lbutils import get_setting

from .models import Book
from .models import Category
from .models import Author
from .forms import BookForm


class DateUtilsTests(TestCase):
    def test_fmt_month(self):
        self.assertEqual('01', fmt_month('1'))
        self.assertEqual('12', fmt_month('12'))

    def test_get_year_choices(self):
        choices = get_year_choices()
        self.assertTrue(choices[0][0])
        choices = get_year_choices(blank_label='------')
        self.assertEqual('', choices[0][0])


class UtilsTests(TestCase):
    def test_fmt_num(self):
        self.assertEqual('9,000,000', fmt_num(9000000))

    def test_create_instance(self):
        d = create_instance('datetime.datetime', year=2000, month=2, day=1)
        self.assertEqual('2000-02-01', d.strftime('%Y-%m-%d'))

    def test_format_filesize(self):
        self.assertEqual('40 B', format_filesize(40))
        self.assertEqual('3.91 KB', format_filesize(4000))
        self.assertEqual('3.81 MB', format_filesize(4000000))


def create_books():
    category = Category.objects.create(name='category-01')
    Category.objects.create(name='category-02')
    author1 = Author.objects.create(name='author-01')
    author2 = Author.objects.create(name='author-02')
    Author.objects.create(name='author-03')
    book = Book.objects.create(
        category=category, name='book-01',
        price=100,
        descn='descn')
    book.authors = [author1, author2]
    Book.objects.create(
        category=category, name='book-02',
        price=200,
        descn='descn')
    Book.objects.create(
        name='book-03',
        is_active=False,
        descn='descn')
    return Book.objects.all()


class QSTests(TestCase):
    def setUp(self):
        create_books()

    def test_get_or_none(self):
        self.assertEqual(None, get_or_none(Book, name='not exist'))
        self.assertEqual(
            'book-01',
            get_or_none(Book, name='book-01').name)

    def test_get_pk_or_none(self):
        self.assertEqual(None, get_pk_or_none(Book, name='not exist'))
        self.assertEqual(
            Book.objects.get(name='book-01').pk,
            get_pk_or_none(Book, name='book-01'))

    def test_get_sum(self):
        qs = Book.objects.all()
        self.assertEqual(300, get_sum(qs, 'price'))
        qs = Book.objects.filter(name='book-01')
        self.assertEqual(100, get_sum(qs, 'price'))

    def test_get_max(self):
        qs = Book.objects.all()
        self.assertEqual(200, get_max(qs, 'price'))

    def test_do_filter_quick_query_fields(self):
        books = Book.objects.all()
        qdata = {'q_quick_search_kw': ''}
        qs = do_filter(books, qdata, ['name', 'category__name'])
        self.assertEqual(3, qs.count())
        qdata = {'q_quick_search_kw': 'ook-01'}
        qs = do_filter(books, qdata, ['name', 'category__name'])
        self.assertEqual(1, qs.count())
        qdata = {'q_quick_search_kw': 'xxxxx'}
        qs = do_filter(books, qdata, ['name', 'category__name'])
        self.assertEqual(0, qs.count())
        qdata = {'q_quick_search_kw': 'catego'}
        qs = do_filter(books, qdata, ['name', 'category__name'])
        self.assertEqual(2, qs.count())

    def test_do_filter_int_quick_query_fields(self):
        books = Book.objects.all()
        qdata = {'q_quick_search_kw': '100'}
        qs = do_filter(books, qdata, ['name', 'category__name'], int_quick_query_fields=['price'])
        self.assertEqual(1, qs.count())

    def test_do_filter(self):
        books = Book.objects.all()
        qdata = {'q__price': '100'}
        qs = do_filter(books, qdata)
        self.assertEqual(1, qs.count())
        qdata = {'q__name__icontains': 'book-03'}
        qs = do_filter(books, qdata)
        self.assertEqual(1, qs.count())
        qdata = {'q__name__icontains': 'ook-02,ook-03'}
        qs = do_filter(books, qdata)
        self.assertEqual(2, qs.count())
        qdata = {'q__name__in': 'book-02,book-03'}
        qs = do_filter(books, qdata)
        self.assertEqual(2, qs.count())
        qdata = {'q__category__name__icontains': 'catego'}
        qs = do_filter(books, qdata)
        self.assertEqual(2, qs.count())
        qdata = {'q__is_active': '__False'}
        qs = do_filter(books, qdata)
        self.assertEqual(1, qs.count())
        # sql: select * from table where field1>field2
        qdata = {'q__name__gt': 'F__name'}
        qs = do_filter(books, qdata)
        self.assertEqual(0, qs.count())


class FormTests(TestCase):
    def setUp(self):
        create_books()
        data = {'name': 'book name'}
        self.form = BookForm(data)
        self.edit_form = BookForm(
            instance=Book.objects.get(name='book-01'))

    @skipUnless(django.VERSION < (1, 10, 0), "crispy_forms not support Django 1.10")
    def test_quicksearchform(self):
        from crispy_forms.templatetags.crispy_forms_filters import as_crispy_form
        data = {'q_quick_search_kw': 'keyword'}
        form = QuickSearchForm(data)
        form.is_valid()
        self.assertEqual('keyword', form.cleaned_data['q_quick_search_kw'])
        self.assertTrue('id_q_quick_search_kw' in as_crispy_form(form))

    def test_filter_fields(self):
        # filter_fields(self, fields=[], exclude=[], include_all_if_empty=True):
        field_names = ['name', 'descn']
        fields = self.form.filter_fields(field_names)
        self.assertEqual(field_names, [e.name for e in fields])
        fields = self.form.filter_fields(exclude=field_names)
        self.assertEqual(['price', 'is_active', 'category', 'authors'], [e.name for e in fields])
        fields = self.form.filter_fields(include_all_if_empty=True)
        self.assertEqual(['name', 'descn', 'price', 'is_active', 'category', 'authors'], [e.name for e in fields])
        fields = self.form.filter_fields(include_all_if_empty=False)
        self.assertEqual([], [e.name for e in fields])

    def test_init_crispy_helper(self):
        self.form.init_crispy_helper()
        self.assertTrue(self.form.helper is not None)

    def test_layout_fields(self):
        self.form.init_crispy_helper()
        self.form.layout_fields([
            ('name', 'price'),
            ('descn', ),
        ])

    @skipUnless(django.VERSION >= (1, 7, 0), "test only applies to Django 1.7+")
    def test_errors_as_text(self):
        data = {}
        form = BookForm(data)
        form.is_valid()
        self.assertTrue('Name' in form.errors_as_text())

    def test_as_text_fields(self):
        self.edit_form.as_text_fields(include_all_if_empty=True)
        self.assertTrue("<span class='text-value'>book-01</span>" in self.edit_form.as_table())

    def test_as_hidden_fields(self):
        self.form.as_hidden_fields(include_all_if_empty=True)
        self.assertTrue('<input id="id_name" name="name" type="hidden" value="book name" />' in self.form.as_table())

    def test_add_attr2fields(self):
        self.form.add_attr2fields('fn', 'fx1')
        self.assertTrue('fn="fx1"' in self.form.as_table())
        self.form.add_attr2fields('fn', 'fx2')
        self.assertTrue('fn="fx1 fx2"' in self.form.as_table())

    def test_as_required_fields(self):
        data = {'name': 'book name'}
        form = BookForm(data)
        self.form.is_valid()
        self.assertTrue('errorlist' not in form.as_table())
        form = BookForm(data)
        self.form.as_required_fields(['descn'])
        self.form.is_valid()
        self.assertTrue('errorlist' not in form.as_table())

    def test_check_uniqe(self):
        book = Book.objects.create(
            name='book-0x',
            price=100,
            descn='descn')
        # existed
        data = {'name': 'book-0x'}
        form = BookForm(data)
        form.is_valid()
        self.assertTrue('errorlist' in form.as_table())
        # edit object
        form = BookForm(data, instance=book)
        form.is_valid()
        self.assertTrue('errorlist' not in form.as_table())
        # not existed
        form = BookForm({'name': 'new book'}, instance=book)
        form.is_valid()
        self.assertTrue('errorlist' not in form.as_table())

    def test_row_div(self):
        self.form.row_div(['name', 'price'], 6)

    def test_just_selected_widgets(self):
        html = self.edit_form.as_table()
        self.assertTrue('category-01' in html)
        self.assertTrue('category-02' not in html)
        self.assertTrue('author-01' in html)
        self.assertTrue('author-02' in html)
        self.assertTrue('author-03' not in html)


class FormSetTests(TestCase):
    pass


class ViewsTests(TestCase):
    def test_forms_is_valid(self):
        sform = QuickSearchForm({})
        bform = BookForm({})
        forms = [sform, bform]
        self.assertFalse(forms_is_valid(forms))
        bform = BookForm({'name': 'ok'})
        forms = [sform, bform]
        self.assertTrue(forms_is_valid(forms))

    def test_render_json(self):
        data = {'name': 'name 中文'}
        out = render_json(data).content
        self.assertEqual(
            b'{"name": "name \xe4\xb8\xad\xe6\x96\x87"}', out)
        # TODO jsonp

    def test_qdict_get_list(self):
        data = QueryDict('a=1&a=2&a=')
        self.assertTrue('' not in qdict_get_list(data, 'a'))


class TagsTests(TestCase):
    def test_display_array(self):
        self.assertEqual('1, 2, 3', display_array(['1', '2', '3']))

    def test_get_setting(self):
        ctx = {}
        v = get_setting(ctx, key='GET_SETTING', default_val="", as_key=None)
        self.assertEqual('ABC', v)
        get_setting(ctx, key='GET_SETTING', default_val="", as_key='new')
        self.assertEqual('ABC', ctx['new'])
        v = get_setting(ctx, key='GET_NOT_EXIST', default_val="DEFAULT", as_key=None)
        self.assertEqual('DEFAULT', v)


class XlsxUtilsTests(TestCase):
    def test_simple_export2xlsx(self):
        def func_data(o):
            return [
                o.name,
                o.price
            ]
        qs = Book.objects.all()
        titles = ['name', 'price']
        response = simple_export2xlsx('fn.xlsx', titles, qs, func_data)
        self.assertTrue(response is not None)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import forms
from django.forms.formsets import BaseFormSet
from django.forms.models import BaseModelFormSet
from django.forms.models import BaseInlineFormSet
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

try:
    from crispy_forms.bootstrap import PrependedText
    from crispy_forms.helper import FormHelper
    from crispy_forms.layout import Layout
    from crispy_forms.bootstrap import StrictButton
    from crispy_forms.layout import Div
except ImportError:
    pass

from .widgets import JustSelectedSelectMultiple
from .widgets import TextWidget


__all__ = (
    'FormHelperMixin', 'BootstrapFormHelperMixin', 'QuickSearchForm', 'LBBaseFormSet',
    'LBBaseModelFormSet', 'LBBaseInlineFormSet', 'row_div',
)


class FormHelperMixin(object):

    def errors_as_text(self):
        """
        only available to Django 1.7+
        """
        errors = []
        errors.append(self.non_field_errors().as_text())
        errors_data = self.errors.as_data()
        for key, value in errors_data.items():
            field_label = self.fields[key].label
            err_descn = ''.join([force_text(e.message) for e in value])
            error = "%s %s" % (field_label, err_descn)
            errors.append(error)
        return ','.join(errors)

    def as_readonly_fields(self, fields=[], exclude=[], include_all_if_empty=True):
        self.add_attr2fields(
            'readonly', 'readonly',
            fields=fields, exclude=exclude, include_all_if_empty=True)

    def as_text_fields(self, fields=[], exclude=[], include_all_if_empty=True):
        for f in self.filter_fields(fields, exclude, include_all_if_empty):
            f = self.fields[f.name]
            f.widget = TextWidget(src_widget=f.widget)

    def as_hidden_fields(self, fields=[], exclude=[], include_all_if_empty=True):
        for f in self.filter_fields(fields, exclude, include_all_if_empty):
            f = self.fields[f.name]
            f.required = False
            if isinstance(f.widget, (forms.SelectMultiple, JustSelectedSelectMultiple)):
                f.widget = forms.MultipleHiddenInput()
            else:
                f.widget = forms.HiddenInput()

    def add_attr2fields(self, attr_name, attr_val, fields=[], exclude=[], include_all_if_empty=True):
        """
        add attr to fields
        """
        for f in self.filter_fields(fields, exclude, include_all_if_empty):
            f = self.fields[f.name]
            org_val = f.widget.attrs.get(attr_name, '')
            f.widget.attrs[attr_name] = '%s %s' % (org_val, attr_val) if org_val else attr_val

    def add_class2fields(self, html_class, fields=[], exclude=[], include_all_if_empty=True):
        """
        add class to html widgets.
        """
        self.add_attr2fields('class', html_class, fields, exclude)

    def filter_fields(self, fields=[], exclude=[], include_all_if_empty=True):
        """
        filter fields

        fields:
        exclude:
        include_all_if_empty: if fields is empty return all fields

        return: fileds
        """
        if not include_all_if_empty and not fields:
            return []
        ret = []
        for f in self.visible_fields():
            if fields and f.name not in fields:
                continue
            if exclude and f.name in exclude:
                continue
            ret.append(f)
        return ret

    def as_required_fields(self, fields=[]):
        """ set required to True """
        fields = self.filter_fields(fields)
        for f in fields:
            f = self.fields[f.name]
            f.required = True

    def check_uniqe(self, obj_class, error_msg=_('Must be unique'), **kwargs):
        """ check if this object is unique """
        if obj_class.objects.filter(**kwargs).exclude(pk=self.instance.pk):
            raise forms.ValidationError(error_msg)

    def row_div(self, fnames, span=4):
        fields = self.filter_fields(fnames)
        fnames = [e.name for e in fields if not e.is_hidden]
        return row_div(fnames, span=span)


class BootstrapFormHelperMixin(FormHelperMixin):

    def init_crispy_helper(self):
        # multicolumn form http://www.bootply.com/s2mmi1YyL4#
        self.helper = helper = FormHelper()
        helper.field_template = 'bootstrap3/multicolumnsfield.html'
        helper.label_class = 'col-md-2'
        helper.field_class = 'col-md-4'
        helper.form_tag = False
        return helper

    def layout_fields(self, grouped_fields):
        divs = []
        one_row_fields = []
        for fields in grouped_fields:
            if len(fields) == 1:
                one_row_fields.extend(fields)
            divs.append(Div(*fields, css_class='form-group'),)
        self.helper.layout = Layout(*divs)
        self.one_row_fields = one_row_fields


class QuickSearchForm(FormHelperMixin, forms.Form):
    q_quick_search_kw = forms.CharField(label=_("Keyword"), required=False)

    def __init__(self, *args, **kw):
        super(QuickSearchForm, self).__init__(*args, **kw)
        self.add_class2fields('input-sm')
        self.helper = helper = FormHelper()
        helper.form_class = 'form-inline'
        helper.form_method = 'get'
        helper.field_template = 'bootstrap3/layout/inline_field.html'
        helper.layout = Layout(
            'q_quick_search_kw',
            StrictButton(_('Search'), type="submit", css_class='btn-sm btn-default'),
        )


class LBBaseFormSet(BaseFormSet):
    """
    Subclass for BaseFormSet.
    Add ext_data support for __init__.
    """

    def __init__(self, *args, **kwargs):
        self.ext_params = {}
        if 'ext_data' in kwargs:
            self.ext_params['ext_data'] = kwargs.pop('ext_data')
        super(LBBaseFormSet, self).__init__(*args, **kwargs)

    def _construct_form(self, i, **kwargs):
        kwargs.update(self.ext_params)
        return super(LBBaseFormSet, self)._construct_form(i, **kwargs)


class LBBaseModelFormSet(BaseModelFormSet):
    """
    Subclass for BaseModelFormSet
    Add ext_data support for __init__.
    """

    def __init__(self, *args, **kwargs):
        self.ext_params = {}
        if 'ext_data' in kwargs:
            self.ext_params['ext_data'] = kwargs.pop('ext_data')
        super(LBBaseModelFormSet, self).__init__(*args, **kwargs)

    def _construct_form(self, i, **kwargs):
        kwargs.update(self.ext_params)
        return super(LBBaseModelFormSet, self)._construct_form(i, **kwargs)


class LBBaseInlineFormSet(BaseInlineFormSet):
    """
    Subclass for BaseInlineFormSet
    Add ext_data support for __init__.
    """

    def __init__(self, *args, **kwargs):
        self.ext_params = {}
        if 'ext_data' in kwargs:
            self.ext_params['ext_data'] = kwargs.pop('ext_data')
        super(LBBaseInlineFormSet, self).__init__(*args, **kwargs)

    def _construct_form(self, i, **kwargs):
        kwargs.update(self.ext_params)
        return super(LBBaseInlineFormSet, self)._construct_form(i, **kwargs)


def row_div(fnames, span=4, hidden_fields=[]):
    divs = []
    for fname in fnames:
        if fname in hidden_fields:
            continue
        if isinstance(fname, PrependedText) and fname.field in hidden_fields:
            continue
        divs.append(
            Div(fname, css_class='col-md-%s form-group' % span),
        )
    return Div(*divs, css_class='row')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from itertools import chain
import django
from django.utils.encoding import force_text
from django.forms.models import ModelChoiceIterator
from django.utils.html import escape
try:
    from django.forms.util import flatatt
except ImportError:  # Django >= 1.9
    from django.forms.utils import flatatt
from django.utils.safestring import mark_safe
from django.utils.datastructures import MultiValueDict
if django.VERSION < (1, 9):
    from django.utils.datastructures import MergeDict
from django.forms import Select
from django.forms import MultipleHiddenInput, HiddenInput
from django.forms.widgets import Widget, Textarea, CheckboxInput

__all__ = (
    'JustSelectedSelect', 'JustSelectedSelectMultiple', 'TextWidget',
)


class JustSelectedSelect(Select):
    """ only generate selected option """

    def render_options(self, choices, selected_choices):
        # Normalize to strings.
        # FIXME selected value couldn't be ‘’ None
        selected_choices = set([force_text(v) for v in selected_choices if v])
        output = []
        schoices = self.choices
        if isinstance(schoices, ModelChoiceIterator):
            schoices.queryset = schoices.queryset.filter(pk__in=selected_choices)
        if isinstance(choices, ModelChoiceIterator):
            choices.queryset = choices.queryset.filter(pk__in=selected_choices)
        for option_value, option_label in chain(schoices, choices):
            if isinstance(option_label, (list, tuple)):
                output.append(u'<optgroup label="%s">' % escape(force_text(option_value)))
                for option in option_label:
                    if not force_text([0]) in selected_choices:
                        continue
                    output.append(self.render_option(selected_choices, *option))
                output.append(u'</optgroup>')
            else:
                if not force_text(option_value) in selected_choices:
                    continue
                output.append(self.render_option(selected_choices, option_value, option_label))
        return u'\n'.join(output)

    def render_readonly(self, name, value, attrs):
        schoices = self.choices
        if isinstance(schoices, ModelChoiceIterator):
            if isinstance(value, list):
                schoices.queryset = schoices.queryset.filter(pk__in=value)
            else:
                schoices.queryset = schoices.queryset.filter(pk=value)
        for o in schoices:
            if "%s" % o[0] == "%s" % value:
                return o[1]
        return ""


class JustSelectedSelectMultiple(JustSelectedSelect):
    """ only generate selected option """

    def render(self, name, value, attrs=None, choices=()):
        if value is None:
            value = []
        final_attrs = self.build_attrs(attrs, name=name)
        output = [u'<select multiple="multiple"%s>' % flatatt(final_attrs)]
        options = self.render_options(choices, value)
        if options:
            output.append(options)
        output.append('</select>')
        return mark_safe(u'\n'.join(output))

    def value_from_datadict(self, data, files, name):
        cls_tuple = (MultiValueDict, )
        if django.VERSION < (1, 9):
            cls_tuple = (MultiValueDict, MergeDict, )
        if isinstance(data, cls_tuple):
            return data.getlist(name)
        return data.get(name, None)

    def _has_changed(self, initial, data):
        if initial is None:
            initial = []
        if data is None:
            data = []
        if len(initial) != len(data):
            return True
        initial_set = set([force_text(value) for value in initial])
        data_set = set([force_text(value) for value in data])
        return data_set != initial_set


def render_hidden(name, value):
    """ render as hidden widget """
    if isinstance(value, list):
        return MultipleHiddenInput().render(name, value)
    return HiddenInput().render(name, value)


class TextWidget(Widget):
    """ render as text """

    def __init__(self, attrs=None, src_widget=None):
        super(TextWidget, self).__init__(attrs)
        self.src_widget = src_widget

    def gen_output(self, descn, value, name):
        return "<span class='text-value'>%s</span> %s" % (descn, render_hidden(name, value))

    def value_from_datadict(self, data, files, name):
        return self.src_widget.value_from_datadict(data, files, name)

    def render(self, name, value, attrs=None):
        func_render_readonly = getattr(self.src_widget, 'render_readonly', None)
        if func_render_readonly:
            descn = func_render_readonly(name, value, attrs)
            return self.gen_output(descn, value, name)
        if isinstance(self.src_widget, Select):
            if isinstance(value, list):
                values = ["%s" % e for e in value]
            else:
                values = ["%s" % value]
            descns = []
            for v, descn in self.src_widget.choices:
                if "%s" % v in values:
                    descns.append(descn)
            return self.gen_output(','.join(descns), value, name)
        if isinstance(self.src_widget, CheckboxInput):
            descn = u'√' if value else u'×'
            return self.gen_output(descn, value, name)
        descn = '' if value is None else '%s' % value
        # TODO if in pre element
        if isinstance(self.src_widget, Textarea):
            descn = value
        return self.gen_output(descn, value, name)



from __future__ import unicode_literals


from datetime import datetime

__all__ = (
    'get_month_choices', 'MONTH_CHOICES', 'fmt_month',
    'get_year_choices', 'fmt_datetime',
)


def get_month_choices(blank_label='------'):
    choices = [
        ('01', '01'),
        ('02', '02'),
        ('03', '03'),
        ('04', '04'),
        ('05', '05'),
        ('06', '06'),
        ('07', '07'),
        ('08', '08'),
        ('09', '09'),
        ('10', '10'),
        ('11', '11'),
        ('12', '12'),
    ]
    if blank_label:
        choices.insert(0, ('', blank_label))
    return choices


MONTH_CHOICES = get_month_choices(None)


def fmt_month(month):
    """ format month(1 to 01) """
    if not month:
        return month
    return ("%s" % month).zfill(2)


def get_year_choices(start_year=-5, end_year=2, blank_label=''):
    year = datetime.today().year
    years = range(year + start_year, year + end_year)
    choices = [(e, e) for e in years]
    if blank_label:
        choices.insert(0, ('', blank_label))
    return choices


def fmt_datetime(d, fmt='', local=True):
    """
    format date with local support
    ``d``: datetime to format
    ``fmt``: format, default is '%Y-%m-%d %H-%M'
    ``local``: format as local time
    """
    if not d:
        return ''
    if local:
        from django.templatetags.tz import localtime
        d = localtime(d)
    if not fmt:
        fmt = '%Y-%m-%d %H-%M'
    return d.strftime(fmt)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import template
from django.conf import settings
from django.utils.safestring import mark_safe

register = template.Library()


@register.simple_tag(takes_context=True)
def get_setting(context, key, default_val="", as_key=None):
    """
    get val form settings and set to context
      {% load lbutils %}
      {% get_setting "key" default_val "as_key" %}
      {{ as_key }}
      if as_key is None, this tag will return val
    """
    if ("%s" % default_val).startswith('$.'):
        default_val = getattr(settings, default_val[2:])
    val = getattr(settings, key, default_val)
    if not as_key:
        return val
    context[as_key] = val
    return ''


@register.filter(name='boolean_icon')
def boolean_icon(v):
    if v:
        return mark_safe('<i class="fa fa-fw fa-check-circle"/>')
    return ''


@register.filter(name='display_array')
def display_array(objs):
    if not objs:
        return ''
    return ', '.join(['%s' % e for e in objs])


@register.simple_tag
def getvars(request, excludes):
    getvars = request.GET.copy()
    excludes = excludes.split(',')
    for p in excludes:
        if p in getvars:
            del getvars[p]
        if len(getvars.keys()) > 0:
            return "&%s" % getvars.urlencode()
        else:
            return ''



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils import six
from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Category(models.Model):
    name = models.CharField(max_length=255)

    def __str__(self):
        return six.text_type(self.name)


@python_2_unicode_compatible
class Author(models.Model):
    name = models.CharField(max_length=255)

    def __str__(self):
        return six.text_type(self.name)


@python_2_unicode_compatible
class Book(models.Model):
    name = models.CharField(max_length=255)
    descn = models.TextField(blank=True)
    price = models.FloatField(null=True, blank=True)
    is_active = models.BooleanField(default=True)

    category = models.ForeignKey(Category, null=True, blank=True)
    authors = models.ManyToManyField(Author, blank=True)

    class Meta:
        ordering = ["name"]
        permissions = (
            ("sft_mgr_book", "manager book"),
        )

    def __str__(self):
        return six.text_type(self.name)



import os
from setuptools import setup, find_packages


def long_desc(root_path):
    FILES = ['README.rst']
    for filename in FILES:
        filepath = os.path.realpath(os.path.join(root_path, filename))
        if os.path.isfile(filepath):
            with open(filepath, mode='r') as f:
                yield f.read()


HERE = os.path.abspath(os.path.dirname(__file__))
long_description = "\n\n".join(long_desc(HERE))


def get_version(root_path):
    with open(os.path.join(root_path, 'lbutils', '__init__.py')) as f:
        for line in f:
            if line.startswith('__version__ ='):
                return line.split('=')[1].strip().strip('"\'')

setup(
    name='django-lbutils',
    version=get_version(HERE),
    description='A set of useful function/tags/filter for Django',
    long_description=long_description,
    author='vicalloy',
    author_email='zbirder@gmail.com',
    url='https://github.com/vicalloy/lbutils',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['Django>=1.6.0'],
    extras_require={
        'django-crispy-forms': [
            'django-crispy-forms>1.4',
        ],
        'xlsxwriter': [
            'xlsxwriter>0.8',
        ]
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Framework :: Django',
    ],
    zip_safe=False,
    tests_require=["Django>=1.6.0"],
    test_suite='runtests.runtests',
    package_data={
        'lbutils': [
            'locale/*/LC_MESSAGES/django.po', 'locale/*/LC_MESSAGES/django.mo'
        ],
    },
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.utils import six
from django.db.models import Q, F
from django.db.models import Sum
from django.db.models import Max


__all__ = (
    'get_or_none', 'get_pk_or_none', 'get_sum',
    'get_max', 'do_filter',
)


def get_or_none(model_class, *args, **kwargs):
    try:
        return model_class.objects.get(*args, **kwargs)
    except Exception:
        return None


def get_pk_or_none(model_class, *args, **kwargs):
    obj = get_or_none(model_class, *args, **kwargs)
    return obj.pk if obj else None


def get_sum(qs, field):
    """
    get sum for queryset.

    ``qs``: queryset
    ``field``: The field name to sum.
    """
    sum_field = '%s__sum' % field
    qty = qs.aggregate(Sum(field))[sum_field]
    return qty if qty else 0


def get_max(qs, field):
    """
    get max for queryset.

    qs: queryset
    field: The field name to max.
    """
    max_field = '%s__max' % field
    num = qs.aggregate(Max(field))[max_field]
    return num if num else 0


def do_filter(qs, qdata, quick_query_fields=[], int_quick_query_fields=[]):
    """
    auto filter queryset by dict.

    qs: queryset need to filter.
    qdata:
    quick_query_fields:
    int_quick_query_fields:
    """
    try:
        qs = qs.filter(
            __gen_quick_query_params(
                qdata.get('q_quick_search_kw'), quick_query_fields,
                int_quick_query_fields)
        )
        q, kw_query_params = __gen_query_params(qdata)
        qs = qs.filter(q, **kw_query_params)
    except:
        import traceback
        traceback.print_exc()
    return qs


def __gen_quick_query_params(value, fields, int_fields):
    q = Q()
    if not value:
        return q
    for field in fields:
        d = {"%s__icontains" % field: value}
        q = q | Q(**d)
    if value.isdigit():
        for f in int_fields:
            d = {f: value}
            q = q | Q(**d)
    return q


def __gen_query_params(qdata):
    q = Q()
    kw_query_params = {}
    for k, v in qdata.items():
        if k.startswith('q__'):
            k = k[3:]
            if not isinstance(v, six.text_type):
                if v is not None:
                    kw_query_params[k] = v
                continue
            if v == '':
                continue
            v = v.replace('，', ',')
            if v.startswith('F__'):
                v = F(v[3:])
            elif k.endswith('__in'):
                v = [e for e in v.split(',') if e]
            elif ',' in v:
                tmp_q = Q()
                v = [e for e in v.split(',') if e]
                for o in v:
                    tmp_q = tmp_q | Q(**{k: o})
                q = q & tmp_q
                continue
            if isinstance(v, six.text_type):
                v = {'__True': True, '__False': False}.get(v, v)
            kw_query_params[k] = v
    return q, kw_query_params



from __future__ import unicode_literals


import codecs
from datetime import datetime
from django.http import HttpResponse
from django.utils.six import BytesIO

try:
    import xlsxwriter as xlwt
except ImportError:
    pass


__all__ = (
    'export_xlsx', 'xlsw_write_row', 'simple_export2xlsx',
)


def export_xlsx(wb, output, fn):
    """
    export as excel
    wb:
    output:
    fn: file name
    """
    wb.close()
    output.seek(0)
    response = HttpResponse(output.read(), content_type="application/vnd.ms-excel")
    cd = codecs.encode('attachment;filename=%s' % fn, 'utf-8')
    response['Content-Disposition'] = cd
    return response


def xlsw_write_row(ws, row_idx, row, fmt=None):
    """
    ws:
    row_idx: row number
    row: a list, data to write
    fmt: format for cell
    """
    for col_idx in range(len(row)):
        ws.write(row_idx, col_idx, row[col_idx], fmt)
    row_idx += 1
    return row_idx


def simple_export2xlsx(filename, titles, qs, func_data):
    """
    export as excel
    filename: file name
    titles: title for this table
    qs: queryset to export
    func_data: a function to format object to list. ex: `lambda o: [o.pk, o.name]`
    """
    output = BytesIO()
    wb = xlwt.Workbook(output)
    ws = wb.add_worksheet(filename)
    header_fmt = wb.add_format()
    header_fmt.set_bg_color('#C4D89E')
    row_idx = 0
    row_idx = xlsw_write_row(ws, row_idx, titles, header_fmt)
    for o in qs:
        row_idx = xlsw_write_row(ws, row_idx, func_data(o))
    fn = '%s-%s.xlsx' % (filename, datetime.now())
    return export_xlsx(wb, output, fn)
